==> atpbar-1.1.0-py3-none-any.whl: atpbar/pickup.py <==
# Tai Sakuma <tai.sakuma@gmail.com>
import os, time
import threading

from .detach import detach_pickup

##__________________________________________________________________||
class ProgressReportPickup(threading.Thread):
    """A pickup of progress reports.

    This class picks up progress reports and presents them.

    Parameters
    ----------
    queue : multiprocessing.Queue
        The queue through which this class receives progress reports
    presentation :
        The presentation of the reports
    """
    def __init__(self, queue, presentation):
        super().__init__(daemon=True)
        # The daemon makes the functions registered at atexit called
        # even if the pickup is still running

        self.queue = queue
        self.presentation = presentation
        self.last_wait_time = 1.0 # [second]
        self.taskids = set()

    def run(self):
        try:
            self._run_until_the_end_order_arrives()
            self._run_until_reports_stop_coming()
        except EOFError:
            pass

    def _run_until_the_end_order_arrives(self):
        end_order_arrived = False
        while True:
            while not self.queue.empty():
                report = self.queue.get()
                if report is None: # the end order
                    end_order_arrived = True
                    continue
                self._process_report(report)
            if end_order_arrived:
                break
            else:
                self._short_sleep()

    def _run_until_reports_stop_coming(self):
        self.last_time = time.time()
        while self.presentation.active():
            if time.time() - self.last_time > self.last_wait_time:
                break
            while not self.queue.empty():
                report = self.queue.get()
                if report is None:
                    continue
                self.last_time = time.time()
                self._process_report(report)
            self._short_sleep()

    def _process_report(self, report):
        self._detach_self_if_necessary(report)
        self.presentation.present(report)

    def _detach_self_if_necessary(self, report):
        if report['taskid'] in self.taskids:
            return
        self.taskids.add(report['taskid'])
        if os.getpid() == report['pid'] and report['in_main_thread']:
            return
        detach_pickup()

    def _short_sleep(self):
        """sleep very briefly

        used to prevent the empty `while` loop from increasing CPU
        loads

        """
        time.sleep(0.001)


##__________________________________________________________________||

==> atpbar-1.1.1-py3-none-any.whl: atpbar/pickup.py <==
# Tai Sakuma <tai.sakuma@gmail.com>
import os, time
import threading

##__________________________________________________________________||
class ProgressReportPickup(threading.Thread):
    """A pickup of progress reports.

    This class picks up progress reports and presents them.

    Parameters
    ----------
    queue : multiprocessing.Queue
        The queue through which this class receives progress reports
    presentation :
        The presentation of the reports
    detach_func : callable
        The function to be called when this class receives a progress
        report from a sub-thread or a sub-process.
    """
    def __init__(self, queue, presentation, detach_func=None):
        super().__init__(daemon=True)
        # The daemon makes the functions registered at atexit called
        # even if the pickup is still running

        self.queue = queue
        self.presentation = presentation
        self.detach_func = detach_func
        self.last_wait_time = 1.0 # [second]
        self.taskids = set()

    def run(self):
        try:
            self._run_until_the_end_order_arrives()
            self._run_until_reports_stop_coming()
        except EOFError:
            pass

    def _run_until_the_end_order_arrives(self):
        end_order_arrived = False
        while True:
            while not self.queue.empty():
                report = self.queue.get()
                if report is None: # the end order
                    end_order_arrived = True
                    continue
                self._process_report(report)
            if end_order_arrived:
                break
            else:
                self._short_sleep()

    def _run_until_reports_stop_coming(self):
        self.last_time = time.time()
        while self.presentation.active():
            if time.time() - self.last_time > self.last_wait_time:
                break
            while not self.queue.empty():
                report = self.queue.get()
                if report is None:
                    continue
                self.last_time = time.time()
                self._process_report(report)
            self._short_sleep()

    def _process_report(self, report):
        self._detach_self_if_necessary(report)
        self.presentation.present(report)

    def _detach_self_if_necessary(self, report):
        if self.detach_func is None:
            return
        if report['taskid'] in self.taskids:
            return
        self.taskids.add(report['taskid'])
        if os.getpid() == report['pid'] and report['in_main_thread']:
            return
        self.detach_func()

    def _short_sleep(self):
        """sleep very briefly

        used to prevent the empty `while` loop from increasing CPU
        loads

        """
        time.sleep(0.001)


##__________________________________________________________________||
